defmodule Tradex.InstrumentsTest do
  use Tradex.DataCase

  alias Tradex.Instruments

  describe "securities" do
    import Tradex.InstrumentsFixtures

    alias Tradex.Instruments.Security

    @invalid_attrs %{name: nil, ticker: nil, wkn: nil, isin: nil, expiration_date: nil, strike_price: nil}

    test "list_securities/0 returns all securities" do
      security = security_fixture()
      assert Instruments.list_securities() == [security]
    end

    test "get_security!/1 returns the security with given id" do
      security = security_fixture()
      assert Instruments.get_security!(security.id) == security
    end

    test "create_security/1 with valid data creates a security" do
      valid_attrs = %{
        name: "some name",
        ticker: "some ticker",
        wkn: "some wkn",
        isin: "some isin",
        expiration_date: ~D[2025-01-27],
        strike_price: "120.5"
      }

      assert {:ok, %Security{} = security} = Instruments.create_security(valid_attrs)
      assert security.name == "some name"
      assert security.ticker == "some ticker"
      assert security.wkn == "some wkn"
      assert security.isin == "some isin"
      assert security.expiration_date == ~D[2025-01-27]
      assert security.strike_price == Decimal.new("120.5")
    end

    test "create_security/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Instruments.create_security(@invalid_attrs)
    end

    test "update_security/2 with valid data updates the security" do
      security = security_fixture()

      update_attrs = %{
        name: "some updated name",
        ticker: "some updated ticker",
        wkn: "some updated wkn",
        isin: "some updated isin",
        expiration_date: ~D[2025-01-28],
        strike_price: "456.7"
      }

      assert {:ok, %Security{} = security} = Instruments.update_security(security, update_attrs)
      assert security.name == "some updated name"
      assert security.ticker == "some updated ticker"
      assert security.wkn == "some updated wkn"
      assert security.isin == "some updated isin"
      assert security.expiration_date == ~D[2025-01-28]
      assert security.strike_price == Decimal.new("456.7")
    end

    test "update_security/2 with invalid data returns error changeset" do
      security = security_fixture()
      assert {:error, %Ecto.Changeset{}} = Instruments.update_security(security, @invalid_attrs)
      assert security == Instruments.get_security!(security.id)
    end

    test "delete_security/1 deletes the security" do
      security = security_fixture()
      assert {:ok, %Security{}} = Instruments.delete_security(security)
      assert_raise Ecto.NoResultsError, fn -> Instruments.get_security!(security.id) end
    end

    test "change_security/1 returns a security changeset" do
      security = security_fixture()
      assert %Ecto.Changeset{} = Instruments.change_security(security)
    end
  end
end
